/* DateTime.cc

    - Date and Time manipulation functions

    Released by Gili Tzabari - [29/06/98]
*/

#include "DateTime"

/************************************************************************
 *                         Time - Member functions                      *
 ************************************************************************/

Time::Time()
{
  set(0);
} //Time.Time()

Time::Time(ULONG time)
{
  set(time);
} //Time.Time(ULONG)

Time::Time(BYTE hour, BYTE minute, BYTE second) throw(out_of_range)
{
  set(hour, minute, second); //EXCEPTION: out_of_range
} //Time.Time(BYTE, BYTE, BYTE)

Time::Time(const Time &target)
{
  set(target);
} //Time.Time(const Time&)

bool Time::operator<(const Time &target) const
{
  return get() < target.get();
} //Time < Time

bool Time::operator==(const Time &target) const
{
  return get()==target.get();
} //Time == Time

bool Time::operator>(const Time &target) const
{
  return !(*this < target || *this == target);
} //Time > Time

bool Time::operator<=(const Time &target) const
{
  return *this < target || *this == target;
} //Time <= Time

bool Time::operator>=(const Time &target) const
{
  return *this > target || *this == target;
} //Time >= Time

bool Time::operator!=(const Time &target) const
{
  return !(*this==target);
} // Time != Time

ifstream &operator>>(ifstream& stream, Time& target)
{
  if (!stream)
    return stream;
  ULONG result;

  stream >> result;
  if (!stream.fail())
    target.set(result);
  return stream;
} // ifstream >> Time

fstream &operator>>(fstream& stream, Time& target)
{
  if (!stream)
    return stream;
  ULONG result;

  stream >> result;
  if (!stream.fail())
    target.set(result);
  return stream;
} // fstream >> Time

ofstream &operator<<(ofstream& stream, Time& source)
{
  if (!stream)
    return stream;
  stream << source.get();
  return stream;
} // ofstream << Time

fstream &operator<<(fstream& stream, Time& source)
{
  if (!stream)
    return stream;
  stream << source.get();
  return stream;
} // fstream << Time

Time Time::operator+(ULONG target) const throw(overflow_error)
{
  Time result(*this);
  result+=target;
  return result;
} //Time+ULONG

Time Time::operator+(const Time &target) const throw(overflow_error)
{
  return *this+target.get();
} //Time+Time

Time Time::operator-(ULONG target) const throw(underflow_error)
{
  Time result(*this);
  result-=target;
  return result;
} //Time-ULONG

Time Time::operator-(const Time &target) const throw(underflow_error)
{
  return *this-target.get();
} //Time-Time

Time &Time::operator++() throw(overflow_error)
{
  ++value;
  if (value==0)
    throw overflow_error("Time::operator++()");
  return *this;
} //++Time

Time Time::operator++(int dummy) throw(overflow_error)
{
  Time result(value);
  ++*this; //EXCEPTION: overflow_error
  return result;
} //Time++

Time &Time::operator--() throw(underflow_error)
{
  --value;
  if (value==max())
    throw underflow_error("Time::operator--()");
  return *this;
} //--Time

Time Time::operator--(int dummy) throw(underflow_error)
{
  Time result(value);
  --*this; //EXCEPTION: underflow_error
  return result;
} //Time--

Time &Time::operator+=(ULONG target) throw(overflow_error)
{
  value+=target;
  if (value<target)
    throw overflow_error("Time::operator+=(ULONG)");
  return *this;
} // Time += ULONG

Time &Time::operator+=(const Time &target) throw(overflow_error)
{
  *this+=target.get(); //EXCEPTION: overflow_error
  return *this;
} // Time += Time

Time &Time::operator-=(ULONG target) throw(underflow_error)
{
  value-=target;
  if ((max()-value)<target)
    throw underflow_error("Time::operator-=(ULONG)");
  return *this;
} //Time -= ULONG

Time &Time::operator-=(const Time &target) throw(underflow_error)
{
  *this-=target.get(); //EXCEPTION: underflow_error
  return *this;
} //Time -= Time

Time &Time::operator=(const Time &target)
{
  if (&target!=this)
    value=target.get();
  return *this;
} // Time = Time

Time::operator ULONG() const
{
  return get();
} // (ULONG) Time

Time &Time::set(ULONG seconds)
{
  value=seconds;
  return *this;
} //Time.set(ULONG)

Time &Time::set(BYTE hour, BYTE minute, BYTE second) throw(out_of_range)
{
  if (second>60 || minute>24 || hour > 24)
    throw out_of_range("Time::set(BYTE, BYTE, BYTE)");
  set(second+minute*60+hour*60*60);
  return *this;
} //Time.set(BYTE, BYTE, BYTE)

Time &Time::set(const Time &target)
{
  *this=target;
  return *this;
} //Time.set(const Time&)

ULONG Time::get() const
{
  return value;
} //Time.get()

const Time &Time::get(BYTE *hour, BYTE *minute, BYTE *second) const
{
  ULONG time=get();

  if (hour)
    *hour=time/(60*60);
  time-=(time/(60*60))*(60*60);
  if (minute)
    *minute=time/60;
  time-=(time/60)*60;
  if (second)
    *second=time;
  return *this;
} //Time.get(BYTE*, BYTE*, BYTE*)

BYTE Time::hour() const
{ 
  BYTE Hour;
  get(&Hour, NULL, NULL);
  return Hour;
} //Time.hour()

BYTE Time::minute() const
{
  BYTE Minute;
  get(NULL, &Minute, NULL);
  return Minute;
} //Time.minute()

BYTE Time::second() const
{
  BYTE Second;
  get(NULL, NULL, &Second);
  return Second;
} //Time.second()

ULONG Time::max() const
{
  ULONG value=0;
  return --value;
} //Time.max()

ULONG Time::elapsed(const Time &newTime) const
{
  return diff(newTime.get(), get());
} //Time.elapsed(const Time&)

bool Time::adjacent(const Time &target) const
{
  return diff(target.get(), get())==1;
} //Time.adjacent(const Time&)

Time::~Time()
{
} //Time.~Time()

/************************************************************************
 *                    TimeRange - Member functions                      *
 ************************************************************************/

TimeRange::TimeRange()
{
  set(0, 0); //length==0
} //TimeRange.TimeRange()

TimeRange::TimeRange(Time time, Time time2) throw(out_of_range)
{
  set(time, time2); //EXCEPTION: out_of_range
} //TimeRange.TimeRange(Time, Time)

TimeRange::TimeRange(const TimeRange &target)
{
  set(target);
} //TimeRange.TimeRange(const TimeRange&)

bool TimeRange::operator<(const Time &target) const throw(invalid_argument)
{
  if (length()==0)
    throw invalid_argument("TimeRange::operator<(const Time&)");
  return last() < target;
} // TimeRange < Time

bool TimeRange::operator<(const TimeRange &target) const throw(invalid_argument)
{
  if (length()==0 || target.length()==0)
    throw invalid_argument("TimeRange::operator<(const TimeRange&)");
  return first() < target.first();
} // TimeRange < TimeRange

bool TimeRange::operator==(const Time &target) const
{
  return contains(target);
} // TimeRange == Time

bool TimeRange::operator==(const TimeRange &target) const
{
  return length()>0 && target.length()>0 && first()==target.first() && last()==target.last();
} // TimeRange == TimeRange

bool TimeRange::operator>(const Time &target) const throw(invalid_argument)
{
  return !(*this < target || *this == target);
} // TimeRange > Time

bool TimeRange::operator>(const TimeRange &target) const throw(invalid_argument)
{
  return !(*this < target || *this == target);
} // TimeRange > TimeRange

bool TimeRange::operator<=(const Time &target) const throw(invalid_argument)
{
  return *this < target || *this == target;
} // TimeRange <= Time

bool TimeRange::operator<=(const TimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target;
} // TimeRange <= TimeRange

bool TimeRange::operator>=(const Time &target) const throw(invalid_argument)
{
  return *this > target || *this == target;
} // TimeRange >= Time

bool TimeRange::operator>=(const TimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target;
} // TimeRange >= TimeRange

bool TimeRange::operator!=(const Time &target) const
{
  return !(*this==target);
} // TimeRange != Time

bool TimeRange::operator!=(const TimeRange &target) const
{
  return !(*this==target);
} // TimeRange != TimeRange

ifstream &operator>>(ifstream& stream, TimeRange& target)
{
  if (!stream)
    return stream;
  Time first, last;

  stream >> first >> last;
  if (!stream.fail())
    target.set(first, last);
  return stream;
} // ifstream >> TimeRange

fstream &operator>>(fstream& stream, TimeRange& target)
{
  if (!stream)
    return stream;
  Time first, last;

  stream >> first >> last;
  if (!stream.fail())
    target.set(first, last);
  return stream;
} // fstream >> TimeRange

ofstream &operator<<(ofstream& stream, TimeRange& source)
{
  if (!stream)
    return stream;
  stream << Time(source.first()) << Time(source.last());
  return stream;
} // ofstream << TimeRange

fstream &operator<<(fstream& stream, TimeRange& source)
{
  if (!stream)
    return stream;
  stream << Time(source.first()) << Time(source.last());
  return stream;
} // fstream << TimeRange

TimeRange TimeRange::operator+(const Time &target) const throw(overflow_error)
{
  TimeRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //TimeRange+Time

TimeRange TimeRange::operator+(const TimeRange &target) const throw(invalid_argument)
{
  TimeRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeRange+TimeRange

TimeRange TimeRange::operator-(const Time &target) const throw(underflow_error)
{
  TimeRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //TimeRange-Time

TimeRange TimeRange::operator-(const TimeRange &target) const throw(invalid_argument)
{
  TimeRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeRange-TimeRange

TimeRange &TimeRange::operator++() throw(overflow_error)
{
  ++_first;
  ++_last;
  if (last() < first())
    throw overflow_error("TimeRange::operator++()");
  return *this;
} // ++TimeRange

TimeRange TimeRange::operator++(int dummy) throw(overflow_error)
{
  TimeRange result(*this);
  ++*this; //EXCEPTION: overflow_error
  return result;
} // TimeRange++

TimeRange &TimeRange::operator--() throw(underflow_error)
{
  --_first;
  --_last;
  if (last() < first())
    throw underflow_error("TimeRange::operator--");
  return *this;
} // --TimeRange

TimeRange TimeRange::operator--(int dummy) throw(underflow_error)
{
  TimeRange result(*this);
  --*this; //EXCEPTION: underflow_error
  return result;
} // TimeRange--

TimeRange &TimeRange::operator+=(const Time &target) throw(overflow_error)
{
  _first+=target; //EXCEPTION: overflow_error
  _last+=target;  //EXCEPTION: overflow_error
  return *this;
} // TimeRange += Time

TimeRange &TimeRange::operator+=(const TimeRange &target) throw(invalid_argument)
{
  if (target.length()==0)
    return *this;
  if (length()==0)
  {
    set(target);
    return *this;
  }
  if (!overlap(target) && !adjacent(target))
    throw invalid_argument("TimeRange::operator+=(const TimeRange&)");
  if (target.first()<first())
    _first=target.first();
  if (target.last()>last())
    _last=target.last();
  return *this;
} // TimeRange += TimeRange

TimeRange &TimeRange::operator-=(const Time &target) throw(underflow_error)
{
  _first-=target; //EXCEPTION: underflow_error
  _last-=target;  //EXCEPTION: underflow_error
  return *this;
} // TimeRange -= Time

TimeRange &TimeRange::operator-=(const TimeRange &target) throw(invalid_argument)
{
  if (internal(target))
    throw invalid_argument("TimeRange::operator-=(const TimeRange&)");
  if (!(overlap(target)))
    return *this;
  if (*this>=target)
    _first=target.last();
  else
    _last=target.first();
  return *this;
} // TimeRange -= TimeRange

TimeRange &TimeRange::operator=(const TimeRange &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // TimeRange = TimeRange

TimeRange &TimeRange::set(const Time &time, const Time &time2) throw(out_of_range)
{
  _first=time;
  _last=time2;
  if (first()>last())
    throw out_of_range("TimeRange::set(const Time&, const Time&)");
  return *this;
} //TimeRange.set(const Time&, const Time&)

TimeRange &TimeRange::set(const TimeRange &target)
{
  set(target.first(), target.last()); //EXCEPTION: out_of_range
  //assuming 'target' is valid, out_of_range should never be thrown
  return *this;
} //TimeRange.set(const TimeRange&)

Time TimeRange::first() const
{
  return _first;
} //TimeRange.first()

Time TimeRange::last() const
{
  return _last;
} //TimeRange.last()

TimeRange TimeRange::intersection(const TimeRange &target) const
{
  TimeRange result; //length()==0

  if (overlap(target))
    result.set(target.first()<=first() ? first() : target.first(),
             target.last()<=last() ? target.last() : last());
  return result;
} //TimeRange.intersection(const TimeRange&)

TimeRange TimeRange::intersection(const vector<TimeRange> &target) const throw(invalid_argument)
{
  TimeRange result;

  for(vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i); //EXCEPTION: invalid_argument
  return result;
} //TimeRange.intersection(const vector<TimeRange>&)

bool TimeRange::boundary(const Time &target) const
{
  return length()>0 && (target==first() || target==last());
} //TimeRange.boundary(const Time&)

bool TimeRange::boundary(const TimeRange &target) const
{
  return target.length()>0 &&
         (boundary(target.first()) || boundary(target.last()-1));
} //TimeRange.boundary(const TimeRange&)

bool TimeRange::adjacent(const Time &target) const
{
  return length()>0 && (first().adjacent(target) || (last()-1).adjacent(target));
} //TimeRange.adjacent(const Time&)

bool TimeRange::adjacent(const TimeRange &target) const
{
  return target.length()>0 && (adjacent(target.first()) || adjacent(target.last()-1));
} //TimeRange.adjacent(const TimeRange&)

bool TimeRange::internal(const Time &target) const
{
  return length()>0 && target>first() && target<last();
} //TimeRange.internal(const Time&)

bool TimeRange::internal(const TimeRange &target) const
{
  return length()>0 && target.first()>first() && target.last()<last();
} //TimeRange.internal(const TimeRange&)

bool TimeRange::overlap(const TimeRange &target) const
{
  return length()>0 && target.length()>0 &&
         (target.first()<=first() && target.last()>=first()) ||
         (target.first()<=last() && target.last()>=last());
} // TimeRange.overlap(const TimeRange&)

bool TimeRange::contains(const Time &target) const
{
  return boundary(target) || internal(target);
} // TimeRange.contains(const Time&)

bool TimeRange::contains(const TimeRange &target) const
{
  return target.length()>0 && contains(target.first()) && contains(target.last()-1);
} // TimeRange.contains(const TimeRange&)

ULONG TimeRange::length() const
{
  return last()-first();
} //TimeRange.length()

TimeRange::~TimeRange()
{
} //TimeRange.~TimeRange()

/************************************************************************
 *                         Date - Member functions                      *
 ************************************************************************/

const BYTE Date::daysInMonth[]= {31, 28, 31,
                                 30, 31, 30,
                                 31, 31, 30,
                                 31, 30, 31};

Date::Date()
{
  set(0);
} //Date.Date()

Date::Date(BYTE days, BYTE months, ULONG years) throw(out_of_range)
{
  set(days, months, years); //EXCEPTION: out_of_range
} //Date.Date(ULONG, BYTE, BYTE)

Date::Date(ULONG days)
{
  set(days);
} //Date.Date(ULONG)

Date::Date(const Date &target)
{
  set(target);
} //Date.Date(const Date&)

bool Date::operator<(const Date &target) const
{
  return get() < target.get();
} //Date < Date

bool Date::operator==(const Date &target) const
{
  return get()==target.get();
} //Date == Date

bool Date::operator>(const Date &target) const
{
  return !(*this == target || *this < target);
} //Date > Date

bool Date::operator<=(const Date &target) const
{
  return *this < target || *this == target;
} //Date <= Date

bool Date::operator>=(const Date &target) const
{
  return *this > target || *this == target;
} //Date >= Date

bool Date::operator!=(const Date &target) const
{
  return !(*this==target);
} // Date != Date

ifstream &operator>>(ifstream& stream, Date& target)
{
  if (!stream)
    return stream;
  ULONG result;

  stream >> result;
  if (!stream.fail())
    target.set(result);
  return stream;
} // ifstream >> Date

fstream &operator>>(fstream& stream, Date& target)
{
  if (!stream)
    return stream;
  ULONG result;

  stream >> result;
  if (!stream.fail())
    target.set(result);
  return stream;
} // fstream >> Date

ofstream &operator<<(ofstream& stream, Date& source)
{
  if (!stream)
    return stream;
  stream << source.get();
  return stream;
} // ofstream << Date

fstream &operator<<(fstream& stream, Date& source)
{
  if (!stream)
    return stream;
  stream << source.get();
  return stream;
} // fstream << Date

Date Date::operator+(ULONG target) const throw(overflow_error)
{
  Date result(*this);
  result+=target;
  return result;
} //Date+ULONG

Date Date::operator+(const Date &target) const throw(overflow_error)
{
  return *this+target.get();
} //Date+Date

Date Date::operator-(ULONG target) const throw(underflow_error)
{
  Date result(*this);
  result-=target;
  return result;
} //Date-ULONG

Date Date::operator-(const Date &target) const throw(underflow_error)
{
  return *this-target.get();
} //Date-Date

Date &Date::operator++() throw(overflow_error)
{
  ++value;
  if (value==0)
    throw overflow_error("Date::operator++()");
  return *this;
} //++Date

Date Date::operator++(int dummy) throw(overflow_error)
{
  Date result(value);
  ++*this; //EXCEPTION: overflow_error
  return result;
} //Date++

Date &Date::operator--() throw(underflow_error)
{
  --value;
  if (value==max())
    throw underflow_error("Date::operator--()");
  return *this;
} //--Date

Date Date::operator--(int dummy) throw(underflow_error)
{
  Date result(value);
  --*this; //EXCEPTION: underflow_error
  return result;
} //Date--

Date &Date::operator+=(ULONG target) throw(overflow_error)
{
  value+=target;
  if (value<target)
    throw overflow_error("Date::operator+=(ULONG)");
  return *this;
} // Date += ULONG

Date &Date::operator+=(const Date &target) throw(overflow_error)
{
  *this+=target.get(); //EXCEPTION: overflow_error
  return *this;
} // Date += Date

Date &Date::operator-=(ULONG target) throw(underflow_error)
{
  value-=target;
  if ((max()-value)<target)
    throw underflow_error("Date::operator-=(ULONG)");
  return *this;
} //Date -= ULONG

Date &Date::operator-=(const Date &target) throw(underflow_error)
{
  *this-=target.get(); //EXCEPTION: underflow_error
  return *this;
} //Date -= Date

Date &Date::operator=(const Date &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // Date = Date

Date::operator ULONG() const
{
  return get();
} // (ULONG) Date

Date &Date::set(ULONG days)
{
  value=days;
  return *this;
} //Date.set(ULONG)

Date &Date::set(BYTE day, BYTE month, ULONG year) throw(out_of_range)
{
  if (month==0 || month>12 || day==0 || day>daysInMonth[month+1])
    throw out_of_range("Date::set(BYTE, BYTE, ULONG)");

  value=year*365;               //years to days
  value+=leaped(year);          //account for leap years
  for (int i=1; i<month-1; ++i)
    value+=daysInMonth[i+1];    //months to days
  value+=day;                   //days to days
  return *this;
} //Date.set(ULONG, BYTE, BYTE)

Date &Date::set(const Date &target)
{
  *this=target;
  return *this;
} //Date.set(const Date&)

ULONG Date::get() const
{
  return value;
} //Date.get()

const Date &Date::get(BYTE *day, BYTE *month, ULONG *year) const
{
  BYTE date=get();
  ULONG _year=date/365,
        _month;

  while (_year*365+leaped(_year)>date) //overshot date
    _year--;
  if (year)
    *year=_year;
  date-=_year*365+leaped(_year);
  for (_month=JAN; date>daysInMonth[_month]; ++_month)
    date-=daysInMonth[_month];
  if (month)
    *month=_month;
  if (day)
    *day=date;
  return *this;
} //Date.get(BYTE, BYTE, ULONG)

BYTE Date::day() const
{
  BYTE Day;
  get(&Day, NULL, NULL);
  return Day;
} //Date.day()

BYTE Date::month() const
{
  BYTE Month;
  get(NULL, &Month, NULL);
  return Month;
} //Date.month()

ULONG Date::year() const
{
  ULONG Year;
  get(NULL, NULL, &Year);
  return Year;
} //Date.year()

ULONG Date::max() const
{
  ULONG value;
  return --value;
} //Date.max()

ULONG Date::elapsed(const Date &newDate) const
{
  return diff(newDate.get(), get());
} //Date.elapsed(Date)

bool Date::leapYear() const
{
  ULONG Year=year();
  if (Year=0)
    return false;

  if (modf(Year/1000, NULL))   //if year==new millenium
    if (modf(Year/4000, NULL)) //it must be divisable by 4000
      return true;
  else
  if (modf(Year/100, NULL))   //if year==new century
    if (modf(Year/400, NULL)) //it must be divisable by 400
      return true;
  else
    if (modf(Year/4, NULL))   //otherwise, it must be divisable by 4
      return true;
  return false;
} //Date.leapYear()

ULONG Date::leaped(ULONG year) const
{
  ULONG sum=0;

  sum+=year/4;
  sum+=year/400-year/100;
  sum+=year/4000-year/1000;
  return sum;
} //Date.leaped()

ULONG Date::leaped() const
{
  return leaped(year());
} //Date.leaped()

bool Date::adjacent(const Date &target) const
{
  return diff(target.get(), get())==1;
} //Date.adjacent(const Date&)

Date::~Date()
{
} //Date.~Date()

/************************************************************************
 *                    DateRange - Member functions                      *
 ************************************************************************/

DateRange::DateRange()
{
  set(0, 0); //length==0
} //DateRange.DateRange()

DateRange::DateRange(Date first, Date last) throw(out_of_range)
{
  set(first, last); //EXCEPTION: out_of_range
} //DateRange.DateRange(Date, Date)

DateRange::DateRange(const DateRange &target)
{
  set(target);
} //DateRange.DateRange(const DateRange&)

bool DateRange::operator<(const Date &target) const throw(invalid_argument)
{
  if (length()==0)
    throw invalid_argument("DateRange::operator<(const Date&)");
  return last()-1 < target;
} // DateRange < Date

bool DateRange::operator<(const DateRange &target) const throw(invalid_argument)
{
  if (length()==0 || target.length()==0)
    throw invalid_argument("DateRange::operator<(const Date&)");
  return last()-1 < target.first();
} // DateRange < DateRange

bool DateRange::operator==(const Date &target) const
{
  return contains(target);
} // DateRange == Date

bool DateRange::operator==(const DateRange &target) const
{
  return length()>0 && target.length()>0 && first()==target.first() && last()==target.last();
} // DateRange == DateRange

bool DateRange::operator>(const Date &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target);
} // DateRange > Date

bool DateRange::operator>(const DateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target);
} // DateRange > DateRange

bool DateRange::operator<=(const Date &target) const throw(invalid_argument)
{
  return *this < target || *this == target;
} // DateRange <= Date

bool DateRange::operator<=(const DateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target;
} // DateRange <= DateRange

bool DateRange::operator>=(const Date &target) const throw(invalid_argument)
{
  return *this > target || *this == target;
} // DateRange >= Date

bool DateRange::operator>=(const DateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target;
} // DateRange >= DateRange

bool DateRange::operator!=(const Date &target) const
{
  return !(*this==target);
} // DateRange != Date

bool DateRange::operator!=(const DateRange &target) const
{
  return !(*this==target);
} // DateRange != DateRange

ifstream &operator>>(ifstream& stream, DateRange& target)
{
  if (!stream)
    return stream;
  Date first, last;

  stream >> first >> last;
  if (!stream.fail())
    target.set(first, last);
  return stream;
} // ifstream >> DateRange

fstream &operator>>(fstream& stream, DateRange& target)
{
  if (!stream)
    return stream;
  Date first, last;

  stream >> first >> last;
  if (!stream.fail())
    target.set(first, last);
  return stream;
} // fstream >> DateRange

ofstream &operator<<(ofstream& stream, DateRange& source)
{
  if (!stream)
    return stream;
  stream << Date(source.first()) << Date(source.last());
  return stream;
} // ofstream << DateRange

fstream &operator<<(fstream& stream, DateRange& source)
{
  if (!stream)
    return stream;
  stream << Date(source.first()) << Date(source.last());
  return stream;
} // fstream << DateRange

DateRange DateRange::operator+(const Date &target) const throw(overflow_error)
{
  DateRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //DateRange+Date

DateRange DateRange::operator+(const DateRange &target) const throw(invalid_argument)
{
  DateRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //DateRange+DateRange

DateRange DateRange::operator-(const Date &target) const throw(underflow_error)
{
  DateRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //DateRange-Date

DateRange DateRange::operator-(const DateRange &target) const throw(invalid_argument)
{
  DateRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //DateRange-DateRange

DateRange &DateRange::operator++() throw(overflow_error)
{
  ++_first;
  ++_last;
  if (last() < first())
    throw overflow_error("DateRange::operator++()");
  return *this;
} // ++DateRange

DateRange DateRange::operator++(int dummy) throw(overflow_error)
{
  DateRange result(*this);
  ++*this; //EXCEPTION: overflow_error
  return result;
} // DateRange++

DateRange &DateRange::operator--() throw(underflow_error)
{
  --_first;
  --_last;
  if (last() < first())
    throw underflow_error("DateRange::operator--");
  return *this;
} // --DateRange

DateRange DateRange::operator--(int dummy) throw(underflow_error)
{
  DateRange result(*this);
  --*this; //EXCEPTION: underflow_error
  return result;
} // DateRange--

DateRange &DateRange::operator+=(const Date &target) throw(overflow_error)
{
  _first+=target; //EXCEPTION: overflow_error
  _last+=target;  //EXCEPTION: overflow_error
  return *this;
} // DateRange += Date

DateRange &DateRange::operator+=(const DateRange &target) throw(invalid_argument)
{
  if (target.length()==0)
    return *this;
  if (length()==0)
  {
    set(target);
    return *this;
  }
  if (!(overlap(target) || adjacent(target)))
    throw invalid_argument("DateRange::operator+=(const DateRange&)");
  if (target.first()<first())
    _first=target.first();
  if (target.last()>last())
    _last=target.last();
  return *this;
} // DateRange += DateRange

DateRange &DateRange::operator-=(const Date &target) throw(underflow_error)
{
  _first-=target; //EXCEPTION: underflow_error
  _last-=target;  //EXCEPTION: underflow_error
  return *this;
} // DateRange -= Date

DateRange &DateRange::operator-=(const DateRange &target) throw(invalid_argument)
{
  if (internal(target))
    throw invalid_argument("DateRange::operator-=(const DateRange&)");
  if (!overlap(target))
    return *this;
  if (*this>=target)
    _first=target.last();
  else
    _last=target.first();
  return *this;
} // DateRange -= DateRange

DateRange &DateRange::operator=(const DateRange &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // DateRange = DateRange

DateRange &DateRange::set(const Date &first, const Date &last) throw(out_of_range)
{             
  _first=first;
  _last=last;
  return *this;
} //DateRange.set(Date, Date)

DateRange &DateRange::set(const DateRange &target)
{
  set(target.first(), target.last());
  return *this;
} //DateRange.set(const DateRange&)

Date DateRange::first() const
{
  return _first;
} //DateRange.first()

Date DateRange::last() const
{
  return _last;
} //DateRange.last()

DateRange DateRange::intersection(const DateRange &target) const
{
  DateRange result; //length==0

  if (overlap(target))
    result.set(target.first()<=first() ? first() : target.first(),
               target.last()<=last() ? target.last() : last());
  return result;
} //DateRange.intersection(const DateRange&)

DateRange DateRange::intersection(const vector<DateRange> &target) const throw(invalid_argument)
{
  DateRange result;

  for(vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i); //EXCEPTION: invalid_argument
  return result;
} //DateRange.intersection(const vector<DateRange>&)

bool DateRange::boundary(const Date &target) const
{
  return length()>0 && (target==first() || target==last());
} //DateRange.boundary(const Date&)

bool DateRange::boundary(const DateRange &target) const
{
  return target.length()>0 &&
         (boundary(target.first()) || boundary(target.last()-1));
} //DateRange.boundary(const DateRange&)

bool DateRange::adjacent(const Date &target) const
{
  return length()>0 && (first().adjacent(target) || (last()-1).adjacent(target));
} //DateRange.adjacent(const Date&)

bool DateRange::adjacent(const DateRange &target) const
{
  return target.length()>0 &&
         (first().adjacent(target.last()-1) || (last()-1).adjacent(target.first()));
} //DateRange.adjacent(const DateRange&)

bool DateRange::internal(const Date &target) const
{
  return length()>0 && target>first() && target<last();
} //DateRange.internal(const Date&)

bool DateRange::internal(const DateRange &target) const
{
  return length()>0 && target.length()>0 &&
         target.first()>first() && target.last()<last();
} //DateRange.internal(const DateRange&)

bool DateRange::overlap(const DateRange &target) const
{
  return length()>0 && target.length()>0 &&
         ((target.first()<=first() && target.last()>=first()) ||
         (target.last()>=last() && target.first()<=last()));
} // DateRange.overlap(const DateRange&)

bool DateRange::contains(const Date &target) const
{
  return boundary(target) || internal(target);
} // DateRange.contains(const Date&)

bool DateRange::contains(const DateRange &target) const
{
  return target.length()>0 && contains(target.first()) && contains(target.last()-1);
} // DateRange.contains(DateRange&)

ULONG DateRange::length() const
{
  return last().get()-first().get();
} //DateRange.length()

ULONG DateRange::leaped() const
{
  if (length()==0)
    return 0;
  return last().leaped()-first().leaped();
} //DateRange.leaped()

DateRange::~DateRange()
{
} //DateRange.~DateRange()

/************************************************************************
 *                  TimeDateRange - Member functions                    *
 ************************************************************************/

TimeDateRange::TimeDateRange()
{
  set(TimeRange(0, 0), DateRange(0, 0)); //length==0
} //TimeDateRange.TimeDateRange()

TimeDateRange::TimeDateRange(const TimeRange &timeRange, const DateRange &dateTable)
{
  set(timeRange, dateTable);
} //TimeDateRange.TimeDateRange(const TimeRange&, const DateRange&)

TimeDateRange::TimeDateRange(const TimeRange &timeRange, const vector<DateRange> &dateTable)
{
  set(timeRange, dateTable);
} //TimeDateRange.TimeDateRange(const TimeRange&, const vector<DateRange>&)

TimeDateRange::TimeDateRange(const TimeDateRange &target)
{
  set(target);
} //TimeDateRange.TimeDateRange(const TimeDateRange&)

TimeDateRange::TimeDateRange(const DateTimeRange &target) throw(invalid_argument)
{
  set(target); //EXCEPTION: invalid_argument
} //TimeDateRange.TimeDateRange(const DateTimeRange&)

bool TimeDateRange::operator<(const Time &target) const throw(invalid_argument)
{
  return getTime() < target; //EXCEPTION: invalid_argument
} // TimeDateRange < Time

bool TimeDateRange::operator<(const Date &target) const throw(invalid_argument)
{
  /* TimeDateRange is not sorted by Dates, therefore we must search the
     entire vector                                                      */
  for (vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // TimeDateRange < Date

bool TimeDateRange::operator<(const TimeRange &target) const throw(invalid_argument)
{
  return getTime() < target; //EXCEPTION: invalid_argument
} // TimeDateRange < TimeRange

bool TimeDateRange::operator<(const DateRange &target) const throw(invalid_argument)
{
  /* TimeDateRange is not sorted by Dates, therefore we must search the
     entire vector                                                      */
  for (vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // TimeDateRange < DateRange

bool TimeDateRange::operator<(const TimeDateRange &target) const throw(invalid_argument)
{
  if (*(getDate().begin())!=*(target.getDate().begin()))
    return *(getDate().begin())<*(target.getDate().begin());
  return getTime() < target.getTime();
} // TimeDateRange < TimeDateRange

bool TimeDateRange::operator<(const DateTimeRange &target) const throw(invalid_argument)
{
  if (*(getDate().begin())!=target.getDate())       //EXCEPTION: invalid_argument
    return *(getDate().begin())< target.getDate();  //EXCEPTION: invalid_argument
  return getTime() < *(target.getTime().begin());   //EXCEPTION: invalid_argument
} // TimeDateRange < DateTimeRange

bool TimeDateRange::operator==(const Time &target) const
{
  return contains(target);
} //TimeDateRange == Time

bool TimeDateRange::operator==(const Date &target) const
{
  return contains(target);
} //TimeDateRange == Date

bool TimeDateRange::operator==(const TimeRange &target) const
{
  return contains(target);
} //TimeDateRange == TimeRange

bool TimeDateRange::operator==(const DateRange &target) const
{
  return contains(target);
} //TimeDateRange == DateRange

bool TimeDateRange::operator==(const TimeDateRange &target) const
{
  return getDate()==target.getDate() &&
         getTime()==target.getTime();
} //TimeDateRange == TimeDateRange

bool TimeDateRange::operator==(const DateTimeRange &target) const
{
  return target.getTime().size()==1 && getDate().size()==1 &&
         getTime()==*(target.getTime().begin()) &&
         *(getDate().begin())==target.getDate();
} //TimeDateRange == DateTimeRange

bool TimeDateRange::operator>(const Time &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > Time

bool TimeDateRange::operator>(const Date &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > Date

bool TimeDateRange::operator>(const TimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > Time

bool TimeDateRange::operator>(const DateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > Date

bool TimeDateRange::operator>(const TimeDateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > TimeDateRange

bool TimeDateRange::operator>(const DateTimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //TimeDateRange > DateTimeRange

bool TimeDateRange::operator<=(const Time &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= Time

bool TimeDateRange::operator<=(const Date &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= Date

bool TimeDateRange::operator<=(const TimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= TimeRange

bool TimeDateRange::operator<=(const DateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= DateRange

bool TimeDateRange::operator<=(const TimeDateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= TimeDateRange

bool TimeDateRange::operator<=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange <= DateTimeRange

bool TimeDateRange::operator>=(const Time &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= Time

bool TimeDateRange::operator>=(const Date &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= Date

bool TimeDateRange::operator>=(const TimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= TimeRange

bool TimeDateRange::operator>=(const DateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= DateRange

bool TimeDateRange::operator>=(const TimeDateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= TimeDateRange

bool TimeDateRange::operator>=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //TimeDateRange >= DateTimeRange

bool TimeDateRange::operator!=(const Time &target) const
{
  return !(*this==target);
} // TimeDateRange != Time

bool TimeDateRange::operator!=(const Date &target) const
{
  return !(*this==target);
} // TimeDateRange != Date

bool TimeDateRange::operator!=(const TimeRange &target) const
{
  return !(*this==target);
} // TimeDateRange != TimeRange

bool TimeDateRange::operator!=(const DateRange &target) const
{
  return !(*this==target);
} // TimeDateRange != DateRange

bool TimeDateRange::operator!=(const TimeDateRange &target) const
{
  return !(*this==target);
} // TimeDateRange != TimeDateRange

bool TimeDateRange::operator!=(const DateTimeRange &target) const
{
  return !(*this==target);
} // TimeDateRange != DateTimeRange

ifstream &operator>>(ifstream& stream, TimeDateRange& target)
{
  if (!stream)
    return stream;
  TimeRange timeRange;
  DateRange dateRange;
  vector<DateRange> dateTable;
  vector<DateRange>::size_type size;

  stream >> timeRange;
  if (!stream)
    return stream;
  stream >> size;
  for (;size>0; --size)
  {
    if (!stream)
      return stream;
    stream >> dateRange;
    if (!stream.fail())
      dateTable.push_back(dateRange);
    else
      return stream;
  }
  target.set(timeRange, dateTable);
  return stream;
} // ifstream >> TimeDateRange

fstream &operator>>(fstream& stream, TimeDateRange& target)
{
  if (!stream)
    return stream;
  TimeRange timeRange;
  DateRange dateRange;
  vector<DateRange> dateTable;
  vector<DateRange>::size_type size;

  stream >> timeRange;
  if (!stream)
    return stream;
  stream >> size;
  for (;size>0; --size)
  {
    if (!stream)
      return stream;
    stream >> dateRange;
    if (!stream.fail())
      dateTable.push_back(dateRange);
    else
      return stream;
  }
  target.set(timeRange, dateTable);
  return stream;
} // fstream >> TimeDateRange

ofstream &operator<<(ofstream& stream, TimeDateRange& source)
{
  if (!stream)
    return stream;
  vector<DateRange> dateTable=source.getDate();
  TimeRange timeRange=source.getTime();

  stream << timeRange;
  if (!stream)
    return stream;
  stream << dateTable.size();
  for (vector<DateRange>::size_type i=0; i<dateTable.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << dateTable[i];
  }
  return stream;
} // ofstream << TimeDateRange

fstream &operator<<(fstream& stream, TimeDateRange& source)
{
  if (!stream)
    return stream;
  vector<DateRange> dateTable=source.getDate();
  TimeRange timeRange=source.getTime();

  if (!stream)
    return stream;
  stream << timeRange;
  if (!stream)
    return stream;
  stream << dateTable.size();
  for (vector<DateRange>::size_type i=0; i<dateTable.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << dateTable[i];
  }
  return stream;
} // fstream << TimeDateRange

TimeDateRange TimeDateRange::operator+(const Time &target) const throw(overflow_error)
{
  TimeDateRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //TimeDateRange+Time

TimeDateRange TimeDateRange::operator+(const Date &target) const throw(overflow_error)
{
  TimeDateRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //TimeDateRange+Date

TimeDateRange TimeDateRange::operator+(const TimeRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange+TimeRange

TimeDateRange TimeDateRange::operator+(const DateRange &target) const
{
  TimeDateRange result(*this);
  result+=target;
  return result;
} //TimeDateRange+DateRange

TimeDateRange TimeDateRange::operator+(const TimeDateRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange+TimeDateRange

TimeDateRange TimeDateRange::operator+(const DateTimeRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange+DateTimeRange

TimeDateRange TimeDateRange::operator-(const Time &target) const throw(underflow_error)
{
  TimeDateRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //TimeDateRange-Time

TimeDateRange TimeDateRange::operator-(const Date &target) const throw(underflow_error)
{
  TimeDateRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //TimeDateRange-Date

TimeDateRange TimeDateRange::operator-(const TimeRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange-TimeRange

TimeDateRange TimeDateRange::operator-(const DateRange &target) const
{
  TimeDateRange result(*this);
  result-=target;
  return result;
} //TimeDateRange-DateRange

TimeDateRange TimeDateRange::operator-(const TimeDateRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange+TimeDateRange

TimeDateRange TimeDateRange::operator-(const DateTimeRange &target) const throw(invalid_argument)
{
  TimeDateRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //TimeDateRange+DateTimeRange

TimeDateRange &TimeDateRange::operator++() throw(overflow_error)
{
  ++timeRange;          //EXCEPTION: overflow_error
  return *this;
} //++TimeDateRange

TimeDateRange TimeDateRange::operator++(int dummy) throw(overflow_error)
{
  TimeDateRange result(*this);
  ++timeRange;          //EXCEPTION: overflow_error
  return result;
} //TimeDateRange++

TimeDateRange &TimeDateRange::operator--() throw(underflow_error)
{
  --timeRange;          //EXCEPTION: underflow_error
  return *this;
} //--TimeDateRange

TimeDateRange TimeDateRange::operator--(int dummy) throw(underflow_error)
{
  TimeDateRange result(*this);
  --timeRange; //EXCEPTION: underflow_error
  return result;
} //TimeDateRange--

TimeDateRange &TimeDateRange::operator+=(const Time &target) throw(overflow_error)
{
  timeRange+=target; //EXCEPTION: overflow_error
  return *this;
} // TimeDateRange += Time

TimeDateRange &TimeDateRange::operator+=(const Date &target) throw(overflow_error)
{
  for(vector<DateRange>::iterator i=dateTable.begin(); i!=dateTable.end(); ++i)
    *i+=target; //EXCEPTION: overflow_error
  return *this;
} // TimeDateRange += Date

TimeDateRange &TimeDateRange::operator+=(const TimeRange &target) throw(invalid_argument)
{
  timeRange+=target; //EXCEPTION: invalid_argument
  return *this;
} // TimeDateRange += TimeRange

TimeDateRange &TimeDateRange::operator+=(const DateRange &target)
{
  if (target.length()==0)
    return *this;
  DateRange old;

  for (vector<DateRange>::iterator source=dateTable.begin(); source!=dateTable.end(); ++source)
    if (source->overlap(target) || source->length()==0)
    {
      *source+=target; //found overlaping DateRange, merge it with SOURCE
      sort(dateTable.begin(), dateTable.end());
      source=find(dateTable.begin(), dateTable.end(), target);
/*
  REASONING:

  1) vector<> is sorted
  2) We attempt to merge SOURCE with _TARGET. SOURCE being constant and
     TARGET being variable (anywhere before or after SOURCE.)
  3) If _TARGET is located after SOURCE, but does not overlap with it, then
     SOURCE will not overlap with any of the elements past _TARGET in the
     vector<>
  4) If TARGET does not overlap with any of the elements before SOURCE, when
     it is merged with SOURCE the resulting DateRange will not overlap with
     any of the elements before SOURCE.
  5) If overlapping does occur, and the elements are merged, it is possible
     for overlap to occur with any element from _TARGET onward
*/
      for(vector<DateRange>::iterator _target=source+1; _target!=getDate().end(); ++_target)
        if (_target->overlap(*source)) //found overlaping DateRanges
        {
          *source+=*_target;
          old=*source;
          dateTable.erase(_target);
          sort(dateTable.begin(), dateTable.end());
          source=find(dateTable.begin(), dateTable.end(), old);
          _target=source; //SOURCE has changed, _TARGET must change according to SOURCE's new position
        }
      return *this;
    } //SOURCE overlaps with TARGET
  dateTable.push_back(target); //target does not overlap with DateRange
  sort(dateTable.begin(), dateTable.end());
  return *this;
} // TimeDateRange += DateRange

TimeDateRange &TimeDateRange::operator+=(const vector<DateRange> &target)
{
  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    *this+=*i;
  return *this;
} // TimeDateRange += vector<DateRange>

TimeDateRange &TimeDateRange::operator+=(const TimeDateRange &target) throw(invalid_argument)
{
  //merge TimeRange
  *this+=target.getTime(); //EXCEPTION: invalid_argument
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    *this+=*i; //merge DateRanges
  return *this;
} // TimeDateRange += TimeDateRange

TimeDateRange &TimeDateRange::operator+=(const DateTimeRange &target) throw(invalid_argument)
{
  if (target.getTime().size()>1)
    throw invalid_argument("TimeDateRange::operator+=(const DateTimeRange&)");
  *this+=*(target.getTime().begin()); //EXCEPTION: invalid_argument
  *this+=target.getDate();
  return *this;
} // TimeDateRange += DateTimeRange

TimeDateRange &TimeDateRange::operator-=(const Time &target) throw(underflow_error)
{
  timeRange-=target; //EXCEPTION: underflow_error
  return *this;
} //TimeDateRange -= Time

TimeDateRange &TimeDateRange::operator-=(const Date &target) throw(underflow_error)
{
  for(vector<DateRange>::iterator i=dateTable.begin(); i!=dateTable.end(); ++i)
    *i-=target; //EXCEPTION: underflow_error
  return *this;
} //TimeDateRange -= Date

TimeDateRange &TimeDateRange::operator-=(const TimeRange &target) throw(invalid_argument)
{
  timeRange-=target; //EXCEPTION: invalid_argument
  return *this;
} // TimeDateRange -= TimeRange

TimeDateRange &TimeDateRange::operator-=(const DateRange &target)
{
  DateRange old;

  for(vector<DateRange>::iterator i=dateTable.begin(); i!=dateTable.end();++i)
    if (i->overlap(target))
    {
      if (i->internal(target))
      {
        //since TARGET is internal we need to split the DateRange into two
        dateTable.push_back(DateRange(target.last(), i->last()));
        *i-=DateRange(target.first(), i->last());
        old=*i;
        sort(dateTable.begin(), dateTable.end());
        //find new position of i
        i=find(dateTable.begin(), dateTable.end(), old);
      }
      else
        *i-=target;
    }
  return *this;
} // TimeDateRange -= DateRange

TimeDateRange &TimeDateRange::operator-=(const TimeDateRange &target) throw(invalid_argument)
{
  *this-=target.getTime();          //EXCEPTION: invalid_argument
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    *this-=*i;                      //remove DateRanges
  return *this;
} //TimeDateRange -= TimeDateRange

TimeDateRange &TimeDateRange::operator-=(const DateTimeRange &target) throw(invalid_argument)
{
  *this-=target.getDate();          //remove DateRanges
  for(vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i) //EXCEPTION: invalid_argument
    *this-=*i;                      //EXCEPTION: invalid_argument
  return *this;
} //TimeDateRange -= DateTimeRange

TimeDateRange &TimeDateRange::operator=(const TimeDateRange &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // TimeDateRange = TimeDateRange

TimeDateRange &TimeDateRange::operator=(const DateTimeRange &target) throw(invalid_argument)
{
  set(target); //EXCEPTION: invalid_argument
  return *this;
} // TimeDateRange = DateTimeRange

TimeRange TimeDateRange::getTime() const
{
  return timeRange;
} //TimeDateRange.getTime()

vector<DateRange> TimeDateRange::getDate() const
{
  return dateTable;
} //TimeDateRange.getDate()

TimeDateRange &TimeDateRange::set(const TimeRange &_timeRange, const DateRange &dateRange)
{
  timeRange=_timeRange;
  dateTable.erase(dateTable.begin(), dateTable.end());
  dateTable.push_back(dateRange);
  return *this;
} //TimeDateRange.set(const TimeRange&, const DateRange&)

TimeDateRange &TimeDateRange::set(const TimeRange &timeRange, const vector<DateRange> &_dateTable)
{
  set(timeRange, *(_dateTable.begin()));
  for (vector<DateRange>::const_iterator i=_dateTable.begin()+1; i!=_dateTable.end(); ++i)
    *this+=*i;
  sort(dateTable.begin(), dateTable.end());
  return *this;
} //TimeDateRange.set(const TimeRange&, const vector<DateRange>&)

TimeDateRange &TimeDateRange::set(const TimeDateRange &target)
{
  set(target.getTime(), target.getDate());
  return *this;
} //TimeDateRange.set(const TimeDateRange&)

TimeDateRange &TimeDateRange::set(const DateTimeRange &target) throw(invalid_argument)
{
  if (target.getTime().size()>1)
    throw invalid_argument("TimeDateRange::set(const DateTimeRange&)");
  set(*(target.getTime().begin()), target.getDate());
  return *this;
} //TimeDateRange.set(const DateTimeRange&)

TimeRange TimeDateRange::intersection(const TimeRange &target) const
{
  return getTime().intersection(target);
} //TimeDateRange.intersection(const TimeRange&)

vector<TimeRange> TimeDateRange::intersection(const vector<TimeRange> &target) const
{
  DateTimeRange result;

  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getTime();
} //TimeDateRange.intersection(const vector<TimeRange>&)

vector<DateRange> TimeDateRange::intersection(const DateRange &target) const
{
  TimeDateRange result;

  for (vector<DateRange>::const_iterator source=getDate().begin(); source!=getDate().end(); ++source)
    result+=source->intersection(target);
  return result.getDate();
} //TimeDateRange.intersection(const DateRange&)

vector<DateRange> TimeDateRange::intersection(const vector<DateRange> &target) const
{
  TimeDateRange result;

  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getDate();
} //TimeDateRange.intersection(const vector<DateRange>&)

TimeDateRange TimeDateRange::intersection(const TimeDateRange &target) const
{
  return TimeDateRange(intersection(target.getTime()), intersection(target.getDate()));
} //TimeDateRange.intersection(const TimeDateRange&)

vector<TimeDateRange> TimeDateRange::intersection(const DateTimeRange &target) const
{
  DateTime result;

  for (vector<DateRange>::const_iterator _source=getDate().begin(); _source!=getDate().end(); ++_source)
    for (vector<TimeRange>::const_iterator _target=target.getTime().begin(); _target!=target.getTime().end(); ++_target)
      result+=TimeDateRange(intersection(*_target), target.intersection(*_source));
  return (vector<TimeDateRange>)result;
} //TimeDateRange.intersection(const DateTimeRange&)

bool TimeDateRange::internal(const Time &target) const
{
  return getTime().internal(target);
} //TimeDateRange.internal(const Time&)

bool TimeDateRange::internal(const Date &target) const
{
  for(vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //TimeDateRange.internal(const Date&)

bool TimeDateRange::internal(const TimeRange &target) const
{
  return getTime().internal(target);
} //TimeDateRange.internal(const TimeRange&)

bool TimeDateRange::internal(const DateRange &target) const
{
  for(vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //TimeDateRange.internal(const DateRange&)

bool TimeDateRange::internal(const TimeDateRange &target) const
{
  if (!internal(target.getTime()))
    return false;
  for (vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    if (!internal(*i))
      return false;
  return true;
} //TimeDateRange.internal(const TimeDateRange&)

bool TimeDateRange::internal(const DateTimeRange &target) const
{
  for (vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    if (!internal(*i))
      return false;
  if (!internal(target.getDate()))
    return false;
  return true;
} //TimeDateRange.internal(const DateTimeRange&)

bool TimeDateRange::overlap(const TimeRange &target) const
{
  return getTime().overlap(target);
} // TimeDateRange.overlap(const TimeRange&)

bool TimeDateRange::overlap(const DateRange &target) const
{
  for(vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // TimeDateRange.overlap(const DateRange&)

bool TimeDateRange::overlap(const TimeDateRange &target) const
{
  if (!overlap(target.getTime()))
    return false;
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    if (overlap(*i))
      return true; //must overlap with at least one of the DateRanges
  return false;
} // TimeDateRange.overlap(const TimeDateRange&)

bool TimeDateRange::overlap(const DateTimeRange &target) const
{
  if (!overlap(target.getDate()))
    return false;
  for(vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    if (overlap(*i))
      return true; //must overlap with at least one of the TimeRanges
  return false;
} // TimeDateRange.overlap(const DateTimeRange&)

bool TimeDateRange::contains(const Time &target) const
{
  return getTime().contains(target);
} // TimeDateRange.contains(const Time&)

bool TimeDateRange::contains(const Date &target) const
{
  return find(getDate().begin(), getDate().end(), target)!=getDate().end();
} // TimeDateRange.contains(const Date&)

bool TimeDateRange::contains(const TimeRange &target) const
{
  return getTime().contains(target);
} // TimeDateRange.contains(const TimeRange&)

bool TimeDateRange::contains(const vector<TimeRange> &target) const
{
  // Must contain ALL TimeRanges
  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (!contains(*i))
      return false;
  return true;
} // TimeDateRange.contains(const vector<TimeRange>&)

bool TimeDateRange::contains(const DateRange &target) const
{
  return target.length()>0 && find(getDate().begin(), getDate().end(), target)!=getDate().end();
} // TimeDateRange.contains(const DateRange&)

bool TimeDateRange::contains(const vector<DateRange> &target) const
{
  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (contains(*i))
      return true;
  return false;
} // TimeDateRange.contains(const vector<DateRange>&)

bool TimeDateRange::contains(const TimeDateRange &target) const
{
  return contains(target.getTime()) && contains(target.getDate());
} // TimeDateRange.contains(const TimeDateRange&)

bool TimeDateRange::contains(const DateTimeRange &target) const
{
  return contains(target.getTime()) && contains(target.getDate());
} // TimeDateRange.contains(const DateTimeRange&)

ULONG TimeDateRange::length() const
{
  ULONG days;

  for (vector<DateRange>::const_iterator i=getDate().begin(); i!=getDate().end(); ++i)
    days+=i->length();
  return getTime().length()*days;
} //TimeDateRange.length()

TimeDateRange::~TimeDateRange()
{
} //TimeDateRange.~TimeDateRange()

/************************************************************************
 *                    DateTimeRange - Member functions                  *
 ************************************************************************/

DateTimeRange::DateTimeRange()
{
  set(DateRange(0, 0), TimeRange(0, 0)); //length==0
} //DateTimeRange.DateTimeRange()

DateTimeRange::DateTimeRange(const DateRange &dateRange, const TimeRange &timeTable)
{
  set(dateRange, timeTable);
} //DateTimeRange.DateTimeRange(const DateRange&, const TimeRange&)

DateTimeRange::DateTimeRange(const DateRange &dateRange, const vector<TimeRange> &timeTable)
{
  set(dateRange, timeTable);
} //DateTimeRange.DateTimeRange(const DateRange&, const vector<TimeRange>&)

DateTimeRange::DateTimeRange(const TimeDateRange &target) throw(invalid_argument)
{
  set(target); //EXCEPTION: invalid_argument
} //DateTimeRange.DateTimeRange(const TimeDateRange&)

DateTimeRange::DateTimeRange(const DateTimeRange &target)
{
  set(target);
} //DateTimeRange.DateTimeRange(const DateTimeRange&)

bool DateTimeRange::operator<(const Time &target) const throw(invalid_argument)
{
  /* DateTimeRange is not sorted by Times, therefore we must search the
     entire vector                                                      */
  for (vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // DateTimeRange < Time

bool DateTimeRange::operator<(const Date &target) const throw(invalid_argument)
{
  return getDate() < target; //EXCEPTION: invalid_argument
} // DateTimeRange < Date

bool DateTimeRange::operator<(const TimeRange &target) const throw(invalid_argument)
{
  /* DateTimeRange is not sorted by Times, therefore we must search the
     entire vector                                                      */
  for (vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // DateTimeRange < TimeRange

bool DateTimeRange::operator<(const DateRange &target) const throw(invalid_argument)
{
  return getDate() < target; //EXCEPTION: invalid_argument
} // DateTimeRange < DateRange

bool DateTimeRange::operator<(const TimeDateRange &target) const throw(invalid_argument)
{
  if (getDate()!=*(target.getDate().begin()))
    return getDate() < *(target.getDate().begin()); //EXCEPTION: invalid_argument
  return *(getTime().begin()) < target.getTime();   //EXCEPTION: invalid_argument
} // DateTimeRange < TimeDateRange

bool DateTimeRange::operator<(const DateTimeRange &target) const throw(invalid_argument)
{
  if (getDate()!=target.getDate())
    return getDate() < target.getDate();                     //EXCEPTION: invalid_argument
  return *(getTime().begin()) < *(target.getTime().begin()); //EXCEPTION: invalid_argument
} // DateTimeRange < DateTimeRange

bool DateTimeRange::operator==(const Time &target) const
{
  return contains(target);
} //DateTimeRange == Time

bool DateTimeRange::operator==(const Date &target) const
{
  return contains(target);
} //DateTimeRange == Date

bool DateTimeRange::operator==(const TimeRange &target) const
{
  return contains(target);
} //DateTimeRange == TimeRange

bool DateTimeRange::operator==(const DateRange &target) const
{
  return contains(target);
} //DateTimeRange == DateRange

bool DateTimeRange::operator==(const TimeDateRange &target) const
{
  return target.getDate().size()==1 && getTime().size()==1 &&
         getDate()==*(target.getDate().begin()) &&
         *(getTime().begin())==target.getTime();
} //DateTimeRange == TimeDateRange

bool DateTimeRange::operator==(const DateTimeRange &target) const
{
  return getDate()==target.getDate() && getTime()==target.getTime(); //EXCEPTION: invalid_argument
} //DateTimeRange == DateTimeRange

bool DateTimeRange::operator>(const Time &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > Time

bool DateTimeRange::operator>(const Date &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > Date

bool DateTimeRange::operator>(const TimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > Time

bool DateTimeRange::operator>(const DateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > Date

bool DateTimeRange::operator>(const TimeDateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > TimeDateRange

bool DateTimeRange::operator>(const DateTimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTimeRange > DateTimeRange

bool DateTimeRange::operator<=(const Time &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= Time

bool DateTimeRange::operator<=(const Date &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= Date

bool DateTimeRange::operator<=(const TimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= TimeRange

bool DateTimeRange::operator<=(const DateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= DateRange

bool DateTimeRange::operator<=(const TimeDateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= TimeDateRange

bool DateTimeRange::operator<=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange <= DateTimeRange

bool DateTimeRange::operator>=(const Time &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= Time

bool DateTimeRange::operator>=(const Date &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= Date

bool DateTimeRange::operator>=(const TimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= TimeRange

bool DateTimeRange::operator>=(const DateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= DateRange

bool DateTimeRange::operator>=(const TimeDateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= TimeDateRange

bool DateTimeRange::operator>=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTimeRange >= DateTimeRange

bool DateTimeRange::operator!=(const Time &target) const
{
  return !(*this==target);
} // DateTimeRange != Time

bool DateTimeRange::operator!=(const Date &target) const
{
  return !(*this==target);
} // DateTimeRange != Date

bool DateTimeRange::operator!=(const TimeRange &target) const
{
  return !(*this==target);
} // DateTimeRange != TimeRange

bool DateTimeRange::operator!=(const DateRange &target) const
{
  return !(*this==target);
} // DateTimeRange != DateRange

bool DateTimeRange::operator!=(const TimeDateRange &target) const
{
  return !(*this==target);
} // DateTimeRange != TimeDateRange

bool DateTimeRange::operator!=(const DateTimeRange &target) const
{
  return !(*this==target);
} // DateTimeRange != DateTimeRange

ifstream &operator>>(ifstream& stream, DateTimeRange& target)
{
  if (!stream)
    return stream;
  TimeRange timeRange;
  DateRange dateRange;
  vector<TimeRange> timeTable;
  vector<TimeRange>::size_type size;

  stream >> dateRange;
  if (!stream)
    return stream;
  stream >> size;
  for (;size>0; --size)
  {
    if (!stream)
      return stream;
    stream >> timeRange;
    if (!stream.fail())
      timeTable.push_back(timeRange);
    else
      return stream;
  }
  target.set(dateRange, timeTable);
  return stream;
} // ifstream >> DateTimeRange

fstream &operator>>(fstream& stream, DateTimeRange& target)
{
  if (!stream)
    return stream;
  TimeRange timeRange;
  DateRange dateRange;
  vector<TimeRange> timeTable;
  vector<TimeRange>::size_type size;

  stream >> dateRange;
  if (!stream)
    return stream;
  stream >> size;
  for (;size>0; --size)
  {
    if (!stream)
      return stream;
    stream >> timeRange;
    if (!stream.fail())
      timeTable.push_back(timeRange);
    else
      return stream;
  }
  target.set(dateRange, timeTable);
  return stream;
} // fstream >> DateTimeRange

ofstream &operator<<(ofstream& stream, DateTimeRange& source)
{
  if (!stream)
    return stream;
  DateRange dateRange=source.getDate();
  vector<TimeRange> timeTable=source.getTime();

  stream << dateRange;
  if (!stream)
    return stream;
  stream << timeTable.size();
  for (vector<TimeRange>::size_type i=0; i<timeTable.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << timeTable[i];
  }
  return stream;
} // ofstream << DateTimeRange

fstream &operator<<(fstream& stream, DateTimeRange& source)
{
  if (!stream)
    return stream;
  DateRange dateRange=source.getDate();
  vector<TimeRange> timeTable=source.getTime();

  stream << dateRange;
  if (!stream)
    return stream;
  stream << timeTable.size();
  for (vector<TimeRange>::size_type i=0; i<timeTable.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << timeTable[i];
  }
  return stream;
} // fstream << DateTimeRange

DateTimeRange DateTimeRange::operator+(const Time &target) const throw(overflow_error)
{
  DateTimeRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //DateTimeRange+Time

DateTimeRange DateTimeRange::operator+(const Date &target) const throw(overflow_error)
{
  DateTimeRange result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //DateTimeRange+Date

DateTimeRange DateTimeRange::operator+(const TimeRange &target) const
{
  DateTimeRange result(*this);
  result+=target;
  return result;
} //DateTimeRange+TimeRange

DateTimeRange DateTimeRange::operator+(const DateRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange+DateRange

DateTimeRange DateTimeRange::operator+(const TimeDateRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange+TimeDateRange

DateTimeRange DateTimeRange::operator+(const DateTimeRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result+=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange+DateTimeRange

DateTimeRange DateTimeRange::operator-(const Time &target) const throw(underflow_error)
{
  DateTimeRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //DateTimeRange-Time

DateTimeRange DateTimeRange::operator-(const Date &target) const throw(underflow_error)
{
  DateTimeRange result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //DateTimeRange-Date

DateTimeRange DateTimeRange::operator-(const TimeRange &target) const
{
  DateTimeRange result(*this);
  result-=target;
  return result;
} //DateTimeRange-TimeRange

DateTimeRange DateTimeRange::operator-(const DateRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange-DateRange

DateTimeRange DateTimeRange::operator-(const TimeDateRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange+TimeDateRange

DateTimeRange DateTimeRange::operator-(const DateTimeRange &target) const throw(invalid_argument)
{
  DateTimeRange result(*this);
  result-=target;       //EXCEPTION: invalid_argument
  return result;
} //DateTimeRange+DateTimeRange

DateTimeRange &DateTimeRange::operator++() throw(overflow_error)
{
  ++dateRange;          //EXCEPTION: overflow_error
  return *this;
} //++DateTimeRange

DateTimeRange DateTimeRange::operator++(int dummy) throw(overflow_error)
{
  DateTimeRange result(*this);
  ++dateRange;          //EXCEPTION: overflow_error
  return result;
} //DateTimeRange++

DateTimeRange &DateTimeRange::operator--() throw(underflow_error)
{
  --dateRange;          //EXCEPTION: underflow_error
  return *this;
} //--DateTimeRange

DateTimeRange DateTimeRange::operator--(int dummy) throw(underflow_error)
{
  DateTimeRange result(*this);
  --dateRange;          //EXCEPTION: underflow_error
  return result;
} //DateTimeRange--

DateTimeRange &DateTimeRange::operator+=(const Time &target) throw(overflow_error)
{
  for(vector<TimeRange>::iterator i=timeTable.begin(); i!=timeTable.end(); ++i)
    *i+=target; //EXCEPTION: overflow_error
  return *this;
} // DateTimeRange += Time

DateTimeRange &DateTimeRange::operator+=(const Date &target) throw(overflow_error)
{
  dateRange+=target; //EXCEPTION: overflow_error
  return *this;
} // DateTimeRange += Date

DateTimeRange &DateTimeRange::operator+=(const TimeRange &target)
{
  if (target.length()==0)
    return *this;
  TimeRange old;

  for (vector<TimeRange>::iterator source=timeTable.begin(); source!=timeTable.end(); ++source)
    if (source->overlap(target) || source->length()==0)
    {
      *source+=target; //found overlaping TimeRange, merge it with SOURCE
      sort(timeTable.begin(), timeTable.end());
      source=find(timeTable.begin(), timeTable.end(), target);
/*
  REASONING:

  1) vector<> is sorted
  2) We attempt to merge SOURCE with _TARGET. SOURCE being constant and
     TARGET being variable (anywhere before or after SOURCE.)
  3) If _TARGET is located after SOURCE, but does not overlap with it, then
     SOURCE will not overlap with any of the elements past _TARGET in the
     vector<>
  4) If TARGET does not overlap with any of the elements before SOURCE, when
     it is merged with SOURCE the resulting TimeRange will not overlap with
     any of the elements before SOURCE.
  5) If overlapping does occur, and the elements are merged, it is possible
     for overlap to occur with any element from _TARGET onward
*/
      for(vector<TimeRange>::iterator _target=source+1; _target!=getTime().end(); ++_target)
        if (_target->overlap(*source)) //found overlaping DateRanges
        {
          *source+=*_target;
          old=*source;
          timeTable.erase(_target);
          sort(timeTable.begin(), timeTable.end());
          source=find(timeTable.begin(), timeTable.end(), old);
          _target=source; //SOURCE has changed, _TARGET must change according to SOURCE's new position
        }
      return *this;
    } //SOURCE overlaps with TARGET
  timeTable.push_back(target); //target does not overlap with TimeRange
  sort(timeTable.begin(), timeTable.end());
  return *this;
} // DateTimeRange += TimeRange

DateTimeRange &DateTimeRange::operator+=(const vector<TimeRange> &target)
{
  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    *this+=*i;
  return *this;
} // DateTimeRange += vector<TimeRange>

DateTimeRange &DateTimeRange::operator+=(const DateRange &target) throw(invalid_argument)
{
  dateRange+=target; //EXCEPTION: invalid_argument
  return *this;
} // DateTimeRange += DateRange

DateTimeRange &DateTimeRange::operator+=(const TimeDateRange &target) throw(invalid_argument)
{
  if (target.getDate().size()>1)
    throw invalid_argument("DateTimeRange::operator+=(const TimeDateRange&)");
  *this+=target.getTime();
  *this+=*(target.getDate().begin()); //EXCEPTION: invalid_argument
  return *this;
} // DateTimeRange += TimeDateRange

DateTimeRange &DateTimeRange::operator+=(const DateTimeRange &target) throw(invalid_argument)
{
  //merge DateRange
  *this+=target.getDate(); //EXCEPTION: invalid_argument
  for(vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    *this+=*i; //merge TimeRanges
  return *this;
} // DateTimeRange += DateTimeRange

DateTimeRange &DateTimeRange::operator-=(const Time &target) throw(underflow_error)
{
  for(vector<TimeRange>::iterator i=timeTable.begin(); i!=timeTable.end(); ++i)
    *i-=target; //EXCEPTION: underflow_error
  return *this;
} //DateTimeRange -= Time

DateTimeRange &DateTimeRange::operator-=(const Date &target) throw(underflow_error)
{
  dateRange-=target; //EXCEPTION: underflow_error
  return *this;
} //DateTimeRange -= Date

DateTimeRange &DateTimeRange::operator-=(const TimeRange &target) throw(invalid_argument)
{
  TimeRange old;

  for(vector<TimeRange>::iterator i=timeTable.begin(); i!=timeTable.end();++i)
    if (i->overlap(target))
    {
      if (i->internal(target))
      {
        //since TARGET is internal we need to split the TimeRange into two
        timeTable.push_back(TimeRange(target.last(), i->last()));
        *i-=TimeRange(target.first(), i->last());
        old=*i;
        sort(timeTable.begin(), timeTable.end());
        //find new position of i
        i=find(timeTable.begin(), timeTable.end(), old);
      }
      else
        *i-=target;
    }
  return *this;
} // DateTimeRange -= TimeRange

DateTimeRange &DateTimeRange::operator-=(const DateRange &target)
{
  dateRange-=target; //EXCEPTION: invalid_argument
  return *this;
} // DateTimeRange -= DateRange

DateTimeRange &DateTimeRange::operator-=(const TimeDateRange &target) throw(invalid_argument)
{
  *this-=target.getTime();          //remove TimeRanges
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    *this-=*i;                      //EXCEPTION: invalid_argument
  return *this;
} //DateTimeRange -= TimeDateRange

DateTimeRange &DateTimeRange::operator-=(const DateTimeRange &target) throw(invalid_argument)
{
  *this-=target.getDate();          //EXCEPTION: invalid_argument
  for(vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    *this-=*i;                      //remove TimeRanges
  return *this;
} //DateTimeRange -= DateTimeRange

DateTimeRange &DateTimeRange::operator=(const TimeDateRange &target) throw(invalid_argument)
{
  set(target); //EXCEPTION: invalid_argument
  return *this;
} // DateTimeRange = TimeDateRange

DateTimeRange &DateTimeRange::operator=(const DateTimeRange &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // DateTimeRange = DateTimeRange

vector<TimeRange> DateTimeRange::getTime() const
{
  return timeTable;
} //DateTimeRange.getTime()

DateRange DateTimeRange::getDate() const
{
  return dateRange;
} //DateTimeRange.getDate()

DateTimeRange &DateTimeRange::set(const DateRange &_dateRange, const TimeRange &timeRange)
{
  dateRange=_dateRange;
  timeTable.erase(timeTable.begin(), timeTable.end());
  timeTable.push_back(timeRange);
  return *this;
} //DateTimeRange.set(const DateRange&, const TimeRange&)

DateTimeRange &DateTimeRange::set(const DateRange &dateRange, const vector<TimeRange> &_timeTable)
{
  set(dateRange, *(_timeTable.begin()));
  for (vector<TimeRange>::const_iterator i=_timeTable.begin()+1; i!=_timeTable.end(); ++i)
    *this+=*i;
  sort(timeTable.begin(), timeTable.end());
  return *this;
} //DateTimeRange.set(const DateRange&, const vector<TimeRange>&)

DateTimeRange &DateTimeRange::set(const TimeDateRange &target) throw(invalid_argument)
{
  if (target.getDate().size()>1)
    throw invalid_argument("DateTimeRange::set(const TimeDateRange&)");
  set(*(target.getDate().begin()), target.getTime());
  return *this;
} //DateTimeRange.set(const TimeDateRange&)

DateTimeRange &DateTimeRange::set(const DateTimeRange &target)
{
  set(target.getDate(), target.getTime());
  return *this;
} //DateTimeRange.set(const DateTimeRange&)

vector<TimeRange> DateTimeRange::intersection(const TimeRange &target) const
{
  DateTimeRange result;

  for (vector<TimeRange>::const_iterator source=getTime().begin(); source!=getTime().end(); ++source)
    result+=source->intersection(target);
  return result.getTime();
} //DateTimeRange.intersection(const TimeRange&)

vector<TimeRange> DateTimeRange::intersection(const vector<TimeRange> &target) const
{
  DateTimeRange result;

  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getTime();
} //DateTimeRange.intersection(const vector<TimeRange>&)

DateRange DateTimeRange::intersection(const DateRange &target) const
{
  return getDate().intersection(target);
} //DateTimeRange.intersection(const DateRange&)

vector<DateRange> DateTimeRange::intersection(const vector<DateRange> &target) const
{
  TimeDateRange result;

  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getDate();
} //DateTimeRange.intersection(const vector<DateRange>&)

vector<DateTimeRange> DateTimeRange::intersection(const TimeDateRange &target) const
{
  DateTime result;

  for (vector<TimeRange>::const_iterator _source=getTime().begin(); _source!=getTime().end(); ++_source)
    for (vector<DateRange>::const_iterator _target=target.getDate().begin(); _target!=target.getDate().end(); ++_target)
      result+=DateTimeRange(intersection(*_target), target.intersection(*_source));
  return result.get();
} //DateTimeRange.intersection(const TimeDateRange&)

DateTimeRange DateTimeRange::intersection(const DateTimeRange &target) const
{
  DateRange _dateTable;
  vector<TimeRange> _timeRange;
  
  _dateTable=intersection(target.getDate());
  _timeRange=intersection(target.getTime());
  return DateTimeRange(_dateTable, _timeRange);
} //DateTimeRange.intersection(const DateTimeRange&)

bool DateTimeRange::internal(const Time &target) const
{
  for(vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTimeRange.internal(const Time&)

bool DateTimeRange::internal(const Date &target) const
{
  return getDate().internal(target);
} //DateTimeRange.internal(const Date&)

bool DateTimeRange::internal(const TimeRange &target) const
{
  for(vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTimeRange.internal(const TimeRange&)

bool DateTimeRange::internal(const DateRange &target) const
{
  return getDate().internal(target);
} //DateTimeRange.internal(const DateRange&)

bool DateTimeRange::internal(const TimeDateRange &target) const
{
  for (vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    if (!internal(*i))
      return false;
  if (!internal(target.getTime()))
    return false;
  return true;
} //DateTimeRange.internal(const TimeDateRange&)

bool DateTimeRange::internal(const DateTimeRange &target) const
{
  if (!internal(target.getDate()))
    return false;
  for (vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    if (!internal(*i))
      return false;
  return true;
} //DateTimeRange.internal(const DateTimeRange&)

bool DateTimeRange::overlap(const TimeRange &target) const
{
  for(vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // DateTimeRange.overlap(const TimeRange&)

bool DateTimeRange::overlap(const DateRange &target) const
{
  return getDate().overlap(target);
} // DateTimeRange.overlap(const DateRange&)

bool DateTimeRange::overlap(const TimeDateRange &target) const
{
  if (!overlap(target.getTime()))
    return false;
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    if (overlap(*i))
      return true; //must overlap with at least one of the DateRanges
  return false;
} // DateTimeRange.overlap(const TimeDateRange&)

bool DateTimeRange::overlap(const DateTimeRange &target) const
{
  if (!overlap(target.getDate()))
    return false;
  for(vector<TimeRange>::const_iterator i=target.getTime().begin(); i!=target.getTime().end(); ++i)
    if (overlap(*i))
      return true; //must overlap with at least one of the TimeRanges
  return false;
} // DateTimeRange.overlap(const DateTimeRange&)

bool DateTimeRange::contains(const Time &target) const
{
  return find(getTime().begin(), getTime().end(), target)!=getTime().end();
} // DateTimeRange.contains(const Time&)

bool DateTimeRange::contains(const Date &target) const
{
  return getDate().contains(target);
} // DateTimeRange.contains(const Date&)

bool DateTimeRange::contains(const TimeRange &target) const
{
  return target.length()>0 && find(getTime().begin(), getTime().end(), target)!=getTime().end();
} // DateTimeRange.contains(const TimeRange&)

bool DateTimeRange::contains(const vector<TimeRange> &target) const
{
  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (contains(*i))
      return true;
  return false;
} // TimeDateRange.contains(const vector<TimeRange>&)

bool DateTimeRange::contains(const DateRange &target) const
{
  return getDate().contains(target);
} // DateTimeRange.contains(const DateRange&)

bool DateTimeRange::contains(const vector<DateRange> &target) const
{
  // Must contain ALL TimeRanges
  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (!contains(*i))
      return false;
  return true;
} // TimeDateRange.contains(const vector<DateRange>&)

bool DateTimeRange::contains(const TimeDateRange &target) const
{
  return contains(target.getTime()) && contains(target.getDate());
} // DateTimeRange.contains(const TimeDateRange&)

bool DateTimeRange::contains(const DateTimeRange &target) const
{
  return contains(target.getTime()) && contains(target.getDate());
} // DateTimeRange.contains(const DateTimeRange&)

ULONG DateTimeRange::length() const
{
  ULONG seconds;

  for (vector<TimeRange>::const_iterator i=getTime().begin(); i!=getTime().end(); ++i)
    seconds+=i->length();
  return getDate().length()*seconds;
} //DateTimeRange.length()

DateTimeRange::~DateTimeRange()
{
} //DateTimeRange.~DateTimeRange()

/************************************************************************
 *                     DateTime - Member functions                      *
 ************************************************************************/

DateTime::DateTime()
{
  set(DateTimeRange()); //length==0
} //DateTime.DateTime()

DateTime::DateTime(const DateTimeRange &_days)
{
  set(_days);
} //DateTime.DateTime(const DateTimeRange&)

DateTime::DateTime(const vector<DateTimeRange> &_days)
{
  set(_days);
} //DateTime.DateTime(const vector<DateTimeRange>&)

DateTime::DateTime(const DateTime &_dateTime)
{
  set(_dateTime);
} //DateTime.DateTime(const DateTime&)

bool DateTime::operator<(const Time &target) const throw(invalid_argument)
{
  /* DateTime is not sorted by Times, therefore we must search the
     entire vector                                                      */
  for (vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // DateTime < Time

bool DateTime::operator<(const Date &target) const throw(invalid_argument)
{
  return *(get().end()-1) < target; //EXCEPTION: invalid_argument
} // DateTime < Date

bool DateTime::operator<(const TimeRange &target) const throw(invalid_argument)
{
  /* DateTime is not sorted by Times, therefore we must search the
     entire vector                                                      */
  for (vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (*i < target) //EXCEPTION: invalid_argument
      return true;
  return false;
} // DateTime < TimeRange

bool DateTime::operator<(const DateRange &target) const throw(invalid_argument)
{
  return *(get().end()-1) < target; //EXCEPTION: invalid_argument
} // DateTime < DateRange

bool DateTime::operator<(const TimeDateRange &target) const throw(invalid_argument)
{
  return *(get().end()-1) < target; //EXCEPTION: invalid_argument
} // DateTime < DateTimeRange

bool DateTime::operator<(const DateTimeRange &target) const throw(invalid_argument)
{
  return *(get().end()-1) < target; //EXCEPTION: invalid_argument
} // DateTime < DateTimeRange

bool DateTime::operator<(const DateTime &target) const throw(invalid_argument)
{
  return *(get().end()-1) < *(target.get().begin()); //EXCEPTION: invalid_argument
} // DateTime < DateTime

bool DateTime::operator==(const Time &target) const
{
  return contains(target);
} //DateTime == Time

bool DateTime::operator==(const Date &target) const
{
  return contains(target);
} //DateTime == Date

bool DateTime::operator==(const TimeRange &target) const
{
  return contains(target);
} //DateTime == TimeRange

bool DateTime::operator==(const DateRange &target) const
{
  return contains(target);
} //DateTime == DateRange

bool DateTime::operator==(const TimeDateRange &target) const
{
  return contains(target);
} //DateTime == TimeDateRange

bool DateTime::operator==(const DateTimeRange &target) const
{
  return contains(target);
} //DateTime == DateTimeRange

bool DateTime::operator==(const DateTime &target) const
{
  return get()==target.get();
} //DateTime == DateTime

bool DateTime::operator>(const Time &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTime > Time

bool DateTime::operator>(const Date &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTime > Date

bool DateTime::operator>(const TimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTime > Time

bool DateTime::operator>(const DateRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTime > Date

bool DateTime::operator>(const DateTimeRange &target) const throw(invalid_argument)
{
  return !(*this < target && *this == target); //EXCEPTION: invalid_argument
} //DateTime > DateTimeRange

bool DateTime::operator<=(const Time &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTime <= Time

bool DateTime::operator<=(const Date &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTime <= Date

bool DateTime::operator<=(const TimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTime <= TimeRange

bool DateTime::operator<=(const DateRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTime <= DateRange

bool DateTime::operator<=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this < target || *this == target; //EXCEPTION: invalid_argument
} //DateTime <= DateTimeRange

bool DateTime::operator>=(const Time &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTime >= Time

bool DateTime::operator>=(const Date &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTime >= Date

bool DateTime::operator>=(const TimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTime >= TimeRange

bool DateTime::operator>=(const DateRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTime >= DateRange

bool DateTime::operator>=(const DateTimeRange &target) const throw(invalid_argument)
{
  return *this > target || *this == target; //EXCEPTION: invalid_argument
} //DateTime >= DateTimeRange

bool DateTime::operator!=(const Time &target) const
{
  return !(*this==target);
} // DateTime != Time

bool DateTime::operator!=(const Date &target) const
{
  return !(*this==target);
} // DateTime != Date

bool DateTime::operator!=(const TimeRange &target) const
{
  return !(*this==target);
} // DateTime != TimeRange

bool DateTime::operator!=(const DateRange &target) const
{
  return !(*this==target);
} // DateTime != DateRange

bool DateTime::operator!=(const TimeDateRange &target) const
{
  return !(*this==target);
} // DateTime != TimeDateRange

bool DateTime::operator!=(const DateTimeRange &target) const
{
  return !(*this==target);
} // DateTime != DateTimeRange

bool DateTime::operator!=(const DateTime &target) const
{
  return !(*this==target);
} // DateTime != DateTime

ifstream &operator>>(ifstream& stream, DateTime& target)
{
  if (!stream)
    return stream;
  DateTimeRange dateTimeRange;
  vector<DateTimeRange> days;
  vector<DateTimeRange>::size_type nDays; //number of days

  stream >> nDays;
  for (;nDays>0; --nDays)
  {
    if (!stream)
      return stream;
    stream >> dateTimeRange;
    if (!stream.fail())
      days.push_back(dateTimeRange);
    else
      return stream;
  }
  target.set(days);
  return stream;
} // ifstream >> DateTime

fstream &operator>>(fstream& stream, DateTime& target)
{
  if (!stream)
    return stream;
  DateTimeRange dateTimeRange;
  vector<DateTimeRange> days;
  vector<DateTimeRange>::size_type nDays; //number of days

  stream >> nDays;
  for (;nDays>0; --nDays)
  {
    if (!stream)
      return stream;
    stream >> dateTimeRange;
    if (!stream.fail())
      days.push_back(dateTimeRange);
    else
      return stream;
  }
  target.set(days);
  return stream;
} // fstream >> DateTime

ofstream &operator<<(ofstream& stream, DateTime& source)
{
  if (!stream)
    return stream;
  DateTimeRange dateTimeRange;
  vector<DateTimeRange> days;

  stream << days.size();
  for (vector<DateTimeRange>::size_type i=0; i<days.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << days[i];
  }
  return stream;
} // ofstream << DateTime

fstream &operator<<(fstream& stream, DateTime& source)
{
  if (!stream)
    return stream;
  DateTimeRange dateTimeRange;
  vector<DateTimeRange> days;

  stream << days.size();
  for (vector<DateTimeRange>::size_type i=0; i<days.size(); ++i)
  {
    if (!stream)
      return stream;
    stream << days[i];
  }
  return stream;
} // fstream << DateTime

DateTime DateTime::operator+(const Time &target) const throw(overflow_error)
{
  DateTime result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //DateTime+Time

DateTime DateTime::operator+(const Date &target) const throw(overflow_error)
{
  DateTime result(*this);
  result+=target;       //EXCEPTION: overflow_error
  return result;
} //DateTime+Date

DateTime DateTime::operator+(const TimeRange &target) const
{
  DateTime result(*this);
  result+=target;
  return result;
} //DateTime+TimeRange

DateTime DateTime::operator+(const DateRange &target) const
{
  DateTime result(*this);
  result+=target;
  return result;
} //DateTime+DateRange

DateTime DateTime::operator+(const TimeDateRange &target) const
{
  DateTime result(*this);
  result+=target;
  return result;
} //DateTime+TimeDateRange

DateTime DateTime::operator+(const DateTimeRange &target) const
{
  DateTime result(*this);
  result+=target;
  return result;
} //DateTime+DateTimeRange

DateTime DateTime::operator+(const DateTime &target) const
{
  DateTime result(*this);
  result+=target;
  return result;
} //DateTime+DateTime

DateTime DateTime::operator-(const Time &target) const throw(underflow_error)
{
  DateTime result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //DateTime-Time

DateTime DateTime::operator-(const Date &target) const throw(underflow_error)
{
  DateTime result(*this);
  result-=target;       //EXCEPTION: underflow_error
  return result;
} //DateTime-Date

DateTime DateTime::operator-(const TimeRange &target) const
{
  DateTime result(*this);
  result-=target;
  return result;
} //DateTime-TimeRange

DateTime DateTime::operator-(const DateRange &target) const
{
  DateTime result(*this);
  result-=target;
  return result;
} //DateTime-DateRange

DateTime DateTime::operator-(const TimeDateRange &target) const
{
  DateTime result(*this);
  result-=target;
  return result;
} //DateTime-TimeDateRange

DateTime DateTime::operator-(const DateTimeRange &target) const
{
  DateTime result(*this);
  result-=target;
  return result;
} //DateTime-DateTimeRange

DateTime DateTime::operator-(const DateTime &target) const
{
  DateTime result(*this);
  result-=target;
  return result;
} //DateTime+DateTime

DateTime &DateTime::operator++() throw(overflow_error)
{
  *this+=(Date)1; //EXCEPTION: overflow_error
  return *this;
} //++DateTime

DateTime DateTime::operator++(int dummy) throw(overflow_error)
{
  DateTime result(get());
  ++*this;        //EXCEPTION: overflow_error
  return result;
} //DateTime++

DateTime &DateTime::operator--() throw(underflow_error)
{
  *this-=(Date)1; //EXCEPTION: underflow_error
  return *this;
} //--DateTime

DateTime DateTime::operator--(int dummy) throw(underflow_error)
{
  DateTime result(get());
  --*this;        //EXCEPTION: underflow_error
  return result;
} //DateTime--

DateTime &DateTime::operator+=(const Time &target) throw(overflow_error)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i+=target;   //EXCEPTION: overflow_error
  return *this;
} // DateTime += Time

DateTime &DateTime::operator+=(const Date &target) throw(overflow_error)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i+=target;   //EXCEPTION: overflow_error
  return *this;
} // DateTime += Date

DateTime &DateTime::operator+=(const TimeRange &target)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i+=target;
  return *this;
} // DateTime += TimeRange

DateTime &DateTime::operator+=(const DateRange &target)
{
  if (target.length()==0)
    return *this;
  DateTimeRange result(target, TimeRange(0, 0));

  for(vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    for(vector<TimeRange>::const_iterator timeTable=source->getTime().begin(); timeTable!=source->getTime().end(); ++timeTable)
      result+=*timeTable;
  *this+=result;
  return *this;
} // DateTime += DateRange

DateTime &DateTime::operator+=(const TimeDateRange &target)
{
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    *this+=DateTimeRange(DateRange(i->first(),i->last()), target.getTime());
  return *this;
} // DateTime += TimeDateRange

DateTime &DateTime::operator+=(const DateTimeRange &target)
{
  if (target.length()==0)
    return *this;
  DateTimeRange old;

  for (vector<DateTimeRange>::iterator source=days.begin(); source!=days.end(); ++source)
    if (source->overlap(target) || source->length()==0)
    {
      *source+=target; //found overlaping DateRange, merge it with SOURCE
      sort(days.begin(), days.end());
      source=find(days.begin(), days.end(), target);
/*
  REASONING:

  1) vector<> is sorted
  2) We attempt to merge SOURCE with _TARGET. SOURCE being constant and
     TARGET being variable (anywhere before or after SOURCE.)
  3) If _TARGET is located after SOURCE, but does not overlap with it, then
     SOURCE will not overlap with any of the elements past _TARGET in the
     vector<>
  4) If TARGET does not overlap with any of the elements before SOURCE, when
     it is merged with SOURCE the resulting DateTimeRange will not overlap
     with any of the elements before SOURCE.
  5) If overlapping does occur, and the elements are merged, it is possible
     for overlap to occur with any element from _TARGET onward
*/
      for(vector<DateTimeRange>::iterator _target=source+1; _target!=days.end(); ++_target)
        if (_target->overlap(*source)) //found overlaping DateRanges
        {
          *source+=*_target;
          old=*source;
          days.erase(_target);
          sort(days.begin(), days.end());
          source=find(days.begin(), days.end(), old);
          _target=source; //SOURCE has changed, _TARGET must change according to SOURCE's new position
        }
      return *this;
    } //SOURCE overlaps with TARGET
  days.push_back(target); //target does not overlap with DateTimeRange
  sort(days.begin(), days.end());
  return *this;
} // DateTime += DateTimeRange

DateTime &DateTime::operator+=(const DateTime &target)
{
  for(vector<DateTimeRange>::const_iterator i=target.get().begin(); i!=target.get().end(); ++i)
    *this+=*i;
  return *this;
} // DateTime += DateTime

DateTime &DateTime::operator-=(const Time &target) throw(underflow_error)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i-=target; //EXCEPTION: underflow_error
  return *this;
} //DateTime -= Time

DateTime &DateTime::operator-=(const Date &target) throw(underflow_error)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i-=target; //EXCEPTION: underflow_error
  return *this;
} //DateTime -= Date

DateTime &DateTime::operator-=(const TimeRange &target)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i-=target;
  return *this;
} // DateTimeRange -= TimeRange

DateTime &DateTime::operator-=(const DateRange &target)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    *i-=target;
  return *this;
} // DateTime -= DateRange

DateTime &DateTime::operator-=(const TimeDateRange &target)
{
  for(vector<DateRange>::const_iterator i=target.getDate().begin(); i!=target.getDate().end(); ++i)
    *this-=DateTimeRange(DateRange(i->first(),i->last()), target.getTime());
  return *this;
} // DateTime -= TimeDateRange

DateTime &DateTime::operator-=(const DateTimeRange &target)
{
  for(vector<DateTimeRange>::iterator i=days.begin(); i!=days.end(); ++i)
    if (i->overlap(target))
    {
      *i-=target;
      return *this;
    }
  return *this;
} // DateTime -= DateTimeRange

DateTime &DateTime::operator-=(const DateTime &target)
{
  for(vector<DateTimeRange>::const_iterator i=target.get().begin(); i!=target.get().end(); ++i)
    *this-=*i;
  return *this;
} // DateTime -= DateTime

DateTime &DateTime::operator=(const DateTime &target)
{
  if (&target!=this)
    set(target);
  return *this;
} // DateTime = DateTime

DateTime::operator vector<TimeDateRange>() const
{
  vector<TimeDateRange> result;
  TimeDateRange addition;

/*
  DateTime "blocks" are assumed to never overlap with one another.
  "DateTimeRange+=" claims it merges TimeRanges, and "DateTime+=" claims
  it merges DateRanges, therefore this assumption should hold true under
  all conditions.
*/
  for (vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    for (vector<TimeRange>::const_iterator _timeRange=i->getTime().begin(); _timeRange!=i->getTime().end(); ++i)
      result.push_back(TimeDateRange(*_timeRange, i->getDate()));
  return result;
} // (vector<TimeDateRange>) DateTime

vector<DateTimeRange> DateTime::get() const
{
  return days;
} //DateTime.get()

DateTime &DateTime::set(const DateTimeRange &target)
{
  days.erase(days.begin(), days.end());
  days.push_back(target);
  return *this;
} //DateTime.set(const DateTimeRange&)

DateTime &DateTime::set(const vector<DateTimeRange> &target)
{
  set(*target.begin());
  for(vector<DateTimeRange>::const_iterator i=target.begin()+1; i!=target.end(); ++i)
    *this+=*i;
  return *this;
} //DateTime.set(const vector<DateTimeRange>&)

DateTime &DateTime::set(const DateTime &target)
{
  set(target.get());
  return *this;
} //DateTime.set(const DateTimeRange&)

vector<TimeRange> DateTime::intersection(const TimeRange &target) const
{
  DateTimeRange result;

  for (vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    result+=source->intersection(target);
  return result.getTime();
} //DateTime.intersection(const TimeRange&)

vector<TimeRange> DateTime::intersection(const vector<TimeRange> &target) const
{
  DateTimeRange result;

  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getTime();
} //DateTime.intersection(const vector<TimeRange>&)

vector<DateRange> DateTime::intersection(const DateRange &target) const
{
  TimeDateRange result;

  for (vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    result+=source->intersection(target);
  return result.getDate();
} //DateTime.intersection(const DateRange&)

vector<DateRange> DateTime::intersection(const vector<DateRange> &target) const
{
  TimeDateRange result;

  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    result+=intersection(*i);
  return result.getDate();
} //DateTime.intersection(const vector<DateRange>&)

DateTime DateTime::intersection(const TimeDateRange &target) const
{
  DateTime result;

  for (vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    result+=source->intersection(target);
  return result;
} //DateTime.intersection(const TimeDateRange&)

DateTime DateTime::intersection(const DateTimeRange &target) const
{
  DateTime result;

  for (vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    result+=source->intersection(target);
  return result;
} //DateTime.intersection(const DateTimeRange&)

DateTime DateTime::intersection(const DateTime &target) const
{
  DateTime result;

  for (vector<DateTimeRange>::const_iterator source=get().begin(); source!=get().end(); ++source)
    result+=target.intersection(*source);
  return result;
} //DateTime.intersection(const DateTime&)

bool DateTime::internal(const Time &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const Time&)

bool DateTime::internal(const Date &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const Date&)

bool DateTime::internal(const TimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const TimeRange&)

bool DateTime::internal(const DateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const DateRange&)

bool DateTime::internal(const TimeDateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const TimeDateRange&)

bool DateTime::internal(const DateTimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->internal(target))
      return true;
  return false;
} //DateTime.internal(const DateTimeRange&)

bool DateTime::internal(const DateTime &target) const
{
  for(vector<DateTimeRange>::const_iterator i=target.get().begin(); i!=target.get().end(); ++i)
    if (!internal(*i))
      return false;
  return true;
} //DateTime.internal(const DateTime&)

bool DateTime::overlap(const TimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // DateTime.overlap(const TimeRange&)

bool DateTime::overlap(const DateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // DateTime.overlap(const DateRange&)

bool DateTime::overlap(const TimeDateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // DateTime.overlap(const TimeDateRange&)

bool DateTime::overlap(const DateTimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->overlap(target))
      return true;
  return false;
} // DateTime.overlap(const DateTimeRange&)

bool DateTime::overlap(const DateTime &target) const
{
  for(vector<DateTimeRange>::const_iterator i=target.get().begin(); i!=target.get().end(); ++i)
    if (!overlap(*i))
      return false;
  return true;
} // DateTime.overlap(const DateTime&)

bool DateTime::contains(const Time &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const Time&)

bool DateTime::contains(const Date &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const Date&)

bool DateTime::contains(const TimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const TimeRange&)

bool DateTime::contains(const vector<TimeRange> &target) const
{
  for (vector<TimeRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (contains(*i))
      return true;
  return false;
} // TimeDate.contains(const vector<TimeRange>&)

bool DateTime::contains(const DateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const DateRange&)

bool DateTime::contains(const vector<DateRange> &target) const
{
  // Must contain ALL TimeRanges
  for (vector<DateRange>::const_iterator i=target.begin(); i!=target.end(); ++i)
    if (!contains(*i))
      return false;
  return true;
} // TimeDate.contains(const vector<DateRange>&)

bool DateTime::contains(const TimeDateRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const TimeDateRange&)

bool DateTime::contains(const DateTimeRange &target) const
{
  for(vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    if (i->contains(target))
      return true;
  return false;
} // DateTime.contains(const DateTimeRange&)

bool DateTime::contains(const DateTime &target) const
{
  for(vector<DateTimeRange>::const_iterator i=target.get().begin(); i!=target.get().end(); ++i)
    if (!contains(*i))
      return false;
  return true;
} // DateTime.contains(const DateTime&)

ULONG DateTime::length() const
{
  ULONG sum=0;

  for (vector<DateTimeRange>::const_iterator i=get().begin(); i!=get().end(); ++i)
    sum+=i->length();
  return sum;
} //DateTime.length()

DateTime::~DateTime()
{
} //DateTime.~DateTime()
